-- com1int for Euphoria 28/10/00
-- IO for com1 with interrupts
-- by Jean Marie Decru  jmdecru@pi.be


-- WARNING: USE AT YOUR OWN RISK
-- I will not be held responible for any damage caused by the use of this file.

---------------------------HOW TO USE-------------------------------------------
-- counter1, counter2, counter3, counter4 are global addresses (counters)
--   CTS        DSR       RI        CD
-- every change at the inputs from low to high increment the respectively counter
-- you can read them with a= peek4u(counterX)   X = number of counter
--
-- preset them with poke4(counterX,value)
--------------------------------------------------------------------------------
-- i = com1_in()
-- reads 4 inputs at once (CTS,DSR,RI,CD)
-- you can separate them by using  and_bits()
-- i.e if and_bits(com1_in(),#80) > 0 then ..   --#80 = CD input  1000 0000 binair
--                                              --#40 = RI input  0100 0000
--                                              --#20 = DSR input 0010 0000                  
--                                              --#10 = CTS input 0001 0000
--                                              -- or a mix of bits
-- the same inputs updates the counters
-- if you connect buttons,relais ... use this function
-- otherwise read the counters because this function is to 'slow'
---------------------------------------------------------------------------------
-- set_out1()   set DTR output high
-- reset_out1() set DTR ouput low
---------------------------------------------------------------------------------
-- set_out2()   set RTS output high
-- reset_out2() set RTS output low       
---------------------------------------------------------------------------------
-- exitcom1() restore the registers in your computer
-- ALLWAYS terminate your program with this function
---------------------------------------------------------------------------------


include machine.e
include ports.e


atom handlerA_address
sequence usual_address
atom code_segment
atom mirror ,dummy
global atom counter1, counter2, counter3, counter4

constant addr = #03F8         -- com1
mirror = #0B                  -- 0000 1011 

-- record the values of the data segment and code segment
atom  segment

segment = allocate(4)
lock_memory(segment, 4)

counter1 = allocate(4)
lock_memory(counter1, 4)
counter2 = allocate(4)
lock_memory(counter2, 4)
counter3 = allocate(4)
lock_memory(counter3, 4)
counter4 = allocate(4)
lock_memory(counter4, 4)

sequence save_segment_code
save_segment_code = {
    #53,   -- push ebx
    #0E,   -- push cs   or #1E push ds -- only a 16-bit value
    #5B,   -- pop ebx  
    #89, #1D} & int_to_bytes(segment) & -- mov segment, ebx
    {#5B,   -- pop ebx
    #C3    -- ret
}       

atom save_segment
save_segment = allocate(length(save_segment_code))
poke(save_segment, save_segment_code)
call(save_segment)                       -- save code segment

-- N.B. Only read the low order two bytes! - See data segment below
code_segment = peek(segment) + 256 * peek(segment+1)

poke(save_segment+1, #1E) 
call(save_segment)                       -- save data segment

-- Handler A  returns from the interrupt
-- after incrementing the counters

sequence handlerA_code
handlerA_code = {
    #1E,       -- push ds  
    #60,       -- pushad
    
    -- restore our data segment value:
    #BB} & 
    
    -- N.B. Only read low-order 2 bytes of segment.
    peek({segment, 2}) & {0,0} &             -- mov ebx, data segment value
   
   {#53,                                     -- push ebx
    #1F,                                     -- pop ds
     
    #BA,#FE,#03,#00,#00,             --    0: mov	edx, #03FE
    #EC,                             --    5: in	al, dx
    #88,#C2,                         --    6: mov 	dl, al
    #24,#11,                         --    8: and	al, #11   CTS
    #3C,#11,                         --    A: cmp	al, #11
    #75,#06,                         --    C: jnz	L1
    #FF,#05}&int_to_bytes(counter1)  --    E: inc	dword ptr [counter1] 
                                     --   14: L1:
  &{#88,#D0,                         --   14: 	mov 	al, dl
    #24,#22,                         --   16: and	al, #22   DSR 
    #3C,#22,                         --   18: cmp	al, #22
    #75,#06,                         --   1A: jnz	L2
    #FF,#05}&int_to_bytes(counter2)  --   1C: inc	dword ptr [counter2] 
                                     --   22: L2:
  &{#88,#D0,                         --   22: 	mov 	al, dl
    #24,#04,                         --   24: and	al, #04   RI 
    #3C,#04,                         --   26: cmp	al, #04    
    #75,#06,                         --   28: jnz	L3
    #FF,#05}&int_to_bytes(counter3)  --   2A: inc	dword ptr [counter3] 
                                     --   30: L3:
  &{#88,#D0,                         --   30: 	mov 	al, dl
    #24,#88,                         --   32: and	al, #88   CD
    #3C,#88,                         --   34: cmp	al, #88
    #75,#06,                         --   36: jnz	L4
    #FF,#05}&int_to_bytes(counter4)  --   38: inc	dword ptr [counter4] 
                                     --   3E: L4:
  &{#BA,#20,#00,#00,#00,             --   3E: 	mov	edx, #20
    #B0,#20,                         --   43: mov	al, #20
    #EE,                             --   45: out	dx, al
  
    #61,       -- popad
    #1F,       -- pop ds 
    #CF        -- iretd   return from interrupt
   }


handlerA_address = allocate(length(handlerA_code))
poke(handlerA_address, handlerA_code)
lock_memory(handlerA_address, length(handlerA_code))
usual_address = get_vector(#0C) -- com1

poke4(counter1, 0)            -- set to zero
poke4(counter2, 0)
poke4(counter3, 0)
poke4(counter4, 0)

--setup UART
Output(0,addr + 3)            --LCR : disable Divisor Latch acces 
Output(mirror,addr + 4)       --MCR: output_lines com1 low
Output(0,addr + 2)            --FCR: disable FIFO mode 
dummy =Input(addr + 6)        --MSR: reset all bits  just by reading them
Output(#08, addr + 1)         --IER: enable  modem status register interrupt

set_vector(#0C, {code_segment, handlerA_address})    -- redirect interrupt
Output(and_bits(Input(#21),#EF),#21) -- enable com1 in interruptcontoller


--you MUST allways exit your program with exitcom1()!!
global procedure exitcom1()
  Output(#00, addr + 1)               --IER: disable interrupts
  Output(or_bits(Input(#21),#10),#21) -- disable com1 in interruptcontoller
  set_vector(#0C, usual_address)      
end procedure


global function com1_in()        
  --  read com1_input pins  bit7 = CD
  --                        bit6 = RI
  --                        bit5 = DSR
  --                        bit4 = CTS
  --                        bit3..0 not used
                    
  return and_bits(Input(addr+6),#F0)   --input byte .. and#F0
end function

-- set or reset com_outputpins DTR, RTS
-- only the two lower bits are significante

global procedure reset_out1()        -- DTR_pin = low
   mirror = or_bits(mirror,#01)      -- or  0000 0001
   Output(mirror,addr + 4)
end procedure

global procedure reset_out2()        -- RTS_pin = low
   mirror = or_bits(mirror,#02)      -- or 0000 0010
   Output(mirror,addr + 4)
end procedure

global procedure set_out1()          -- DTR_pin = high 
   mirror = and_bits(mirror,#0A)     -- and 0000 1010 
   Output(mirror,addr + 4)
end procedure

global procedure set_out2()           --RTS_pin = high
   mirror = and_bits(mirror,#09)      --and 0000 1001
   Output(mirror,addr + 4)
end procedure






